# Generated by Django 2.0.8 on 2018-09-20 03:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('facebook_login', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='facebookaccount',
            name='fb_user_id',
            field=models.CharField(max_length=1024, unique=True),
        ),
    ]
